# Synced from ballistica-master-server.
# EFRO_SYNC_HASH=132661122838838768130682575592127667134
#
"""Communication from bamaster to basn."""

from __future__ import annotations

from typing import TYPE_CHECKING, Annotated
from dataclasses import dataclass

from typing_extensions import override
from efro.message import Message, Response, MessageProtocol
from efro.dataclassio import ioprepped, IOAttrs

if TYPE_CHECKING:
    pass


def get_protocol() -> MessageProtocol:
    """Get the protocol used in this communication.

    Remember to run 'make meta' after any changes here to update the
    auto-generated sender/receiver classes.
    """

    return MessageProtocol(
        message_types={
            1: AccountPruneInfoRequestMessage,
            2: AccountDeleteMessage,
            45: LoginInfoRequestMessage,
        },
        response_types={
            1: AccountPruneInfoRequestResponse,
            2: AccountDeleteResponse,
            46: LoginInfoRequestResponse,
        },
        remote_errors_include_stack_traces=True,
    )


@ioprepped
@dataclass
class LoginInfoRequestMessage(Message):
    """Request info about a login."""

    # The login we want info about.
    login_id: Annotated[str, IOAttrs('l')]

    # Return its name?
    fetch_name: Annotated[bool, IOAttrs('n')] = False

    # Return stats?
    fetch_stats: Annotated[bool, IOAttrs('s')] = False

    @override
    @classmethod
    def get_response_types(cls) -> list[type[Response] | None]:
        return [LoginInfoRequestResponse]


@ioprepped
@dataclass
class LoginInfoRequestResponse(Response):
    """Info returned from a login info request."""

    account_id: Annotated[str | None, IOAttrs('a')]

    name: Annotated[str | None, IOAttrs('n')] = None

    tickets: Annotated[int | None, IOAttrs('t')] = None
    profile_count: Annotated[int | None, IOAttrs('p')] = None
    purchase_count: Annotated[int | None, IOAttrs('pc')] = None
    all_logins: Annotated[list[str] | None, IOAttrs('l')] = None


@ioprepped
@dataclass
class AccountPruneInfoRequestMessage(Message):
    """Request info about pruning an account."""

    # The login we want info about.
    login_id: Annotated[str, IOAttrs('l')]

    @override
    @classmethod
    def get_response_types(cls) -> list[type[Response] | None]:
        return [AccountPruneInfoRequestResponse]


@ioprepped
@dataclass
class AccountPruneInfoRequestResponse(Response):
    """Info returned from an account-prune-info request."""

    can_prune: Annotated[bool, IOAttrs('p')]


@ioprepped
@dataclass
class AccountDeleteMessage(Message):
    """Delete a v1 account."""

    # The login we want info about.
    login_id: Annotated[str, IOAttrs('l')]

    @override
    @classmethod
    def get_response_types(cls) -> list[type[Response] | None]:
        return [AccountDeleteResponse]


@ioprepped
@dataclass
class AccountDeleteResponse(Response):
    """Info returned from an account-prune-info request."""

    success: Annotated[bool, IOAttrs('s')]
