# Synced from ballistica-master-server.
# EFRO_SYNC_HASH=325374651437076003851060940609981011471
#
"""Bootstrapping communication from basn to bamaster."""

from __future__ import annotations

from typing import TYPE_CHECKING, Annotated
from dataclasses import dataclass

from typing_extensions import override
from efro.dataclassio import ioprepped, IOAttrs
from efro.message import Message, Response, MessageProtocol

if TYPE_CHECKING:
    pass


def get_protocol() -> MessageProtocol:
    """Get the protocol used in this communication.

    Remember to run 'make meta' after any changes here to update the
    auto-generated sender/receiver classes.
    """

    return MessageProtocol(
        message_types={
            14: CommKeyMessage,
        },
        response_types={
            7: CommKeyResponse,
        },
        remote_errors_include_stack_traces=True,
    )


@ioprepped
@dataclass
class CommKeyMessage(Message):
    """For sending initial comm key."""

    key: Annotated[str, IOAttrs('k')]
    address: Annotated[str, IOAttrs('a')]
    port: Annotated[int, IOAttrs('p')]
    version: Annotated[int, IOAttrs('v')]
    cadata: Annotated[str | None, IOAttrs('c')] = None

    @override
    @classmethod
    def get_response_types(cls) -> list[type[Response] | None]:
        return [CommKeyResponse]


@ioprepped
@dataclass
class CommKeyResponse(Response):
    """Response to a sent comm-key message."""

    success: Annotated[bool, IOAttrs('s')]
