"""Task class for test task."""

from __future__ import annotations

import asyncio
from typing import TYPE_CHECKING

from typing_extensions import override
from baserver.enums import TaskTestType

from basn.taskwrapper import (
    StandaloneTaskWrapper,
    InlineTaskWrapper,
    SandboxTaskWrapper,
)

if TYPE_CHECKING:
    from basntask.command import TaskCommand


class TestTaskWrapperStandalone(StandaloneTaskWrapper):
    """Task we can use for testing various things."""

    def __init__(self, test_type: TaskTestType) -> None:
        super().__init__()
        self._test_type = test_type

    @override
    def get_process_priority(self) -> StandaloneTaskWrapper.ProcessPriority:
        # We don't have any planes to catch.
        return StandaloneTaskWrapper.ProcessPriority.LOW

    @override
    def get_label(self) -> str:
        return f'{self._test_type.name.lower()} test'

    @override
    def get_command(self) -> TaskCommand:
        from basntask.testtask import TestTaskCommand

        return TestTaskCommand(self._test_type)


class TestTaskWrapperInline(InlineTaskWrapper):
    """Task we can use for testing various things."""

    def __init__(self, test_type: TaskTestType) -> None:
        super().__init__()
        self._test_type = test_type

    @override
    def get_label(self) -> str:
        return f'{self._test_type.name.lower()} test'

    @override
    async def run_inline(self) -> None:
        self.add_note('Hello from run-inline begin.')

        if self._test_type is TaskTestType.INLINE_ERROR:
            raise RuntimeError('Testing Inline Error')

        await asyncio.sleep(10.0)
        self.add_note('Hello from run-inline end.')


class TestTaskWrapperSandbox(SandboxTaskWrapper):
    """Task we can use for testing various things."""

    def __init__(self, test_type: TaskTestType) -> None:
        super().__init__()
        self._test_type = test_type

    @override
    def get_label(self) -> str:
        return f'{self._test_type.name.lower()} test'
